
package prstatlib;

use SNMP;

$SNMP::auto_init_mib = 0;
$SNMP::verbose = 0;
$SNMP::debugging = 0;

SNMP::addMibDirs( "/homes1/info-e/printer/lib/mibs" );

$NPADMIN="/homes1/info-e/printer/bin/npadmin";
$PRINTER_DAT_PATH="/homes1/info-e/printer/lib/printer.dat";
$LPQ="/usr/ucb/lpq";
$LPC="/usr/ucb/lpc";

#@PRINTCAP= split( "\n", `ypcat -k printcap` );

@PRINTER_DAT = ();
%PRINTER_DAT = ();

# Ermittelt alle Drucker die auf gegeb. Wildcards passen
# Args: 1: Referenz auf Array fuer Treffer (canonical)
#       2: Array mit Pattern
sub match_printer
{
    my $i, $j, $hit, @p;
    my $printer = $_[0];
    my @printer_pat = $_[1];
    my %matching;

    foreach $j ( @printer_pat ){    
        $j =~ s/(.*)/(^|\\s|[,#])(\1)(\$|\\s|[,#])/;
        $j =~ s/\*/\\S*/g;
     }

    foreach $i ( keys( %PRINTER_DAT ) ){
        $hit = 0;
        foreach $j ( @printer_pat ){
            $hit = 1 if( $i =~ /$j/ );
            @p = @{$PRINTER_DAT{$i}};
            $hit = 1 if( $p[4] =~ /$j/ );
            }
        $matching{"$i"} = 1 if( $hit );
        }

    @$printer = keys( %matching );
}

# Args:    1: Druckername
# Returns: (ref hash, ref array)           
#          Hash:  canonical => ( server, interface, typ, filter, options, raum, rmte, acct, comment, hidden )
#                                  0         1       2     3        4       5    6     7      8        9
#          Array: Druckernamen und Kommentare wie in PRINTER_DAT
sub get_printer_info
{
    return ( \%PRINTER_DAT, \@PRINTER_DAT );
}

# Args:    1: Options wie in printer.dat
# Returns  Array mit Aliases
sub parse_aliases
{
    my @options = split( /#/, $_[0] );
    @options = split( ",", $options[ $options[0] ] );
    shift( @options );
    return @options;
}

########
# Status-Abfrage fuer HPs

$HP_DISPLAY_ENTRY = ".1.3.6.1.4.1.11.2.3.9.1.1.3";
$HP_STATUS_ENTRY = ".1.3.6.1.4.1.11.2.3.9.1.1.2";
$HP_STATUS_DUMMY = ".1.3.6.1.4.1.11.2.3.9.1.1.2.1";
$HP_STATUS_ALL = [
    ["$HP_STATUS_ENTRY.1", 0 ],
    ["$HP_STATUS_ENTRY.2", 0 ],
    ["$HP_STATUS_ENTRY.3", 0 ],
    ["$HP_STATUS_DUMMY",   0 ],
    ["$HP_STATUS_ENTRY.5", 0 ],
    ["$HP_STATUS_ENTRY.6", 0 ],
    ["$HP_STATUS_DUMMY",   0 ],
    ["$HP_STATUS_ENTRY.8", 0 ],
    ["$HP_STATUS_ENTRY.9", 0 ],
    ["$HP_STATUS_ENTRY.10", 0 ],
    ["$HP_STATUS_DUMMY",   0 ],
    ["$HP_STATUS_ENTRY.12", 0 ],
    ["$HP_STATUS_ENTRY.13", 0 ],
    ["$HP_STATUS_ENTRY.14", 0 ],
    ["$HP_STATUS_ENTRY.15", 0 ],
    ["$HP_STATUS_ENTRY.16", 0 ],
    ["$HP_STATUS_ENTRY.17", 0 ],
    ["$HP_STATUS_ENTRY.18", 0 ],
    ["$HP_STATUS_ENTRY.19", 0 ],
    
    ["$HP_DISPLAY_ENTRY", 0 ]
    ];

# Umsetzung der gdStatusEntries in Statusbeschreibung
# Reihenfolge ist wichtig: Erster Treffer zaehlt
# Format:
#   SNMP-Byte
#   Treffer falls >=x
#   Beschreibung
#   Dringlichkeit( 0 gruen, 1 gelb, 2 rot, 3 nichts )

@HP_STATUS = (
    [  2, 1, "Papierproblem", 2 ],
    [  5, 1, "Warten...", 0 ],
    [  6, 1, "Fehler", 2 ],
    [  8, 2, "Manuelle Papierzufuherung benoetigt", 1 ],
    [  8, 1, "Kein Papier", 2 ],
    [  9, 1, "Papierstau", 2 ],
    [ 10, 2, "Keine Toner-Cartridge", 2 ],
    [ 10, 1, "Wenig Toner", 1 ],
    [ 12, 1, "Zu wenig Speicher", 2 ],
    [ 13, 1, "Empfaengt", 0 ],
    [ 15, 1, "Bereit", 0 ],
    [ 16, 2, "Aufwaermen", 0 ],
    [ 16, 1, "Inititializierung", 0 ],
    [ 17, 1, "Geraet offen", 0 ],
    [ 19, 1, "Problem bei Papierausgabe", 0 ],
    [  1, 1, "Offline", 2 ],
    [  3, 1, "Service benoetigt", 2 ],
    [ 14, 1, "Druckt...", 0 ],
    [ 18, 1, "Druckt", 0 ],
    );

########
# Status-Abfrage nach RFC 1759

$DISPLAY_ENTRY     = ".1.3.6.1.2.1.43.16.5.1.2.1";
$DEV_STATUS_ENTRY  = ".1.3.6.1.2.1.25.3.2.1.5.1";
$PRT_STATUS_ENTRY  = ".1.3.6.1.2.1.25.3.5.1.1.1";

# Format: hrDeviceStatus, hrPrinterStatus (0: beliebig), Beschreibung, Dringlichkeit
# (nach RFC 1759)
@DEV_STATUS = (
    [ 2, 3, "Bereit", 0 ],
    [ 2, 4, "Druckt", 0 ], 
    [ 3, 3, "Bereit (Warnung)", 1 ], 
    [ 3, 4, "Druckt (Warnung)", 1 ], 
    [ 5, 0, "Fehler", 2 ],
    [ 5, 5, "Aufwaermen", 0 ],
    [ 2, 1, "Standby", 0 ],
    [ 1, 0, "Unbekannt", 1 ],
    [ 2, 0, "Bereit", 0 ],
    [ 3, 0, "Warnung", 1 ],
    [ 4, 0, "Testet", 1 ],
    );

# Funktioniert bei Phasern anscheinend nicht richtig, daher z.Z. leider unbenutzt
$ERROR_STATE_ENTRY = ".1.3.6.1.2.1.25.3.5.1.2.1";
# Bit, Beschreibung
@ERROR_STATE_DESCR = (
    [ 0, "Wenig Papier" ],
    [ 1, "Kein Papier" ],
    [ 2, "Wenig Toner" ],
    [ 3, "Kein Toner" ],
    [ 4, "Geraet offen" ],
    [ 5, "Papierstau" ],
    [ 6, "Offline" ],
    [ 7, "Service noetig" ]
    );

# Holt nur Status und Display
# Args:   1: Druckername (canonical)
#         2: Druckertyp
# Return: (Statustext, Dringlichkeit (0,..,2), Displaytext)

sub get_snmp_short_info {

    my $printer, $name, $p, $i, $j, $stat, @stat;
    my $name = $_[0];
    my $typ = $_[1];
    my $session;
    my $disp;
    my $err, $txt;


    $session = new SNMP::Session( DestHost => $name,
                                  Community => "public",
                                  Retries => 1) ;


    return( "Nicht erreichbar", 2, "" ) if( ! $session );

    # Spezielle Behandlung fuer HPs
    if( $typ =~ /^HP/ ){
    
        @stat = $session->get( $HP_STATUS_ALL );
        
#        foreach $i ( @stat ){
#            print STDERR "$i|";
#        }
#        print STDERR "\n";        

        return( "Nicht erreichbar", 2, "" ) if( ! ( @stat && $stat[0] ne "" ) );

        foreach $i (@HP_STATUS){
            return( @{$i}[2], @{$i}[3], $stat[ $#stat ]  )
                if( $stat[ @{$i}[0] - 1 ] >= @{$i}[1]  );
            }

        return ( "Bereit", 0,  $stat[ $#stat ] );
        }
        
    # Ansonsten nach Printer-MIB
    else{        

        $dev = $session->get( "$DEV_STATUS_ENTRY" );
        $prt = $session->get( "$PRT_STATUS_ENTRY" );

        return( "Nicht erreichbar", 2, "" ) if( $dev eq "" );

        @i = [ "$DISPLAY_ENTRY", 0 ];
        while( 1 ){
            $j = $session->getnext( @i );
            last if( ! ( $i[0][0] =~ $DISPLAY_ENTRY ) );
            $disp .= $j;
        }

        foreach $i (@DEV_STATUS) {
            return( @{$i}[2], @{$i}[3], $disp )
                if( @{$i}[0] == $dev && ( @{$i}[1] == 0 || @{$i}[1] == $prt ) );
            }

        return ( "Unbekannt", 1, $disp );

#        @i = [ "$DISPLAY_ENTRY", 0 ];
#        while( 1 ){
#            $j = $session->getnext( @i );
#            last if( ! ( $i[0][0] =~ $DISPLAY_ENTRY ) );
#            $disp .= $j;
#        }

#        $a = $session->get( ".1.3.6.1.2.1.25.3.5.1.2.1"  );
        
#        @b = unpack( "cccc", $a );
#        printf STDERR "!%d|%d!%d|%d|\n", $b[0], $b[1], $b[2], $b[3];

#        $stat = $session->get( ".1.3.6.1.2.1.12.1.1.5.1" );
#        print STDERR "|stat|$stat|\n";
        
#        @err = $session->get( ".1.3.6.1.2.1.25.3.5.1.2.1" );
        

#        foreach $i (@ERROR_STATE_DESCR){
#            if( $err & ( 1 << @{$i}[0] ) ){
#                print STDERR "(" . @{$i}[1] . "\n";
#                return ( @{$i}[1], @{$i}[2], $disp );
#                }
#           }

#        return ( $DEV_STATUS[ $stat ][0], $DEV_STATUS[ $stat ][1], $disp );
        return ( "Nicht bekannt", $DEV_STATUS[ $stat ][1], $disp );
        }
        
}

# Args:   1: Druckername (canonical)
#         2: Referenz auf Hash fuer Daten 

sub get_snmp_info {

    my $name = $_[0];
    my $data = $_[1];
    my @trays, $i, $count;

    # Ping-Test
    system( "/usr/sbin/ping $name 2 >/dev/null 2>&1" );
    if(  $? >> 8 != 0 ){
#       $$data{"status"} = "ping failed";    
        return;
        }


    # Single Line Options

    $options =   "--name --vendor --model --netconfig --printmib "
               . "--hostmib --memory --duplex --status --pagecount " 
               . "--colors --description --contact";

    open( NPA, "$NPADMIN $options $name 2>/dev/null|" ) || die "Can't start npadmin";

    while( <NPA> ){

        next if /^$/;
        s/\n//;
        
        split( '="|";' );
        while ( @_ ){
            $$data{$_[0]} = $_[1];

            # Parse --description for firmware
            if( $_[0] eq "description" ){
            
                $$data{"firmware"} = "$1 ($2: $3)" 
                    if( $_[1] =~ /^HP .*ROM ([^,]+).*(JD..).*EEPROM ([^,]+)/ );
                $$data{"firmware"} = "$1" 
                    if(  $_[1] =~ /^Tektronix .*firmware version ([^,]*)/ );

            }
            
            shift; shift;
            }
    }   

    # Multiline Options
    get_multi( $name, "display", "%s", "displayBufferText", "", $data );
    get_multi( $name, "cover", "%s", "status", ", ", $data );
    get_multi( $name, "inputtray", "%s: %s", "description,curLevel", ", ", $data );

    $$data{"display"} =~ s/^\s*(.*)\s*$/\1/g;
    $$data{"display"} =~ s/\s+/ /g;

    $count = 1;
    foreach $i ( split( ",", $$data{"inputtray"} ) ){
        @trays = split( ":", $i );
        while( @trays ){
            shift @trays;
            $trays[0] =~ s/ //g;
            $$data{"tray$count"} = $trays[0];;
            shift @trays;
            $count++;
            }
        }

}

# Args: 1: Drucker
#       2: Option
#       3: Ausgabe-Format
#       4: Benoetigte Attribute
#       5: Referenz auf Hash fuer Daten

sub get_multi
{
    my $name = $_[0];
    my $option = $_[1];
    my $format = $_[2];
    my @attr = split( ",", $_[3] );
    my $seperator = $_[4];
    my $data = $_[5];
    my $p = "";
    my @vals, $pars, $i, $name;

    $$data{$option} = "";

    open( NPA, "$NPADMIN --$option $name 2>/dev/null|" ) || die "Can't start npadmin";
    while( <NPA> ){
    
        next if /^$/;
        s/\n//;

        split( '="|";' );
        
        @vals = ();
        while ( @_ ){
            for( $i = 0; $i <= $#attr; $i++ ){
                $vals[$i] = "$_[1]" if( $_[1] ne "" && $_[0] eq $attr[$i] );
                }
            shift; shift;
            }
        
        if( $#vals >= 0 ){
            $$data{$option} .= $seperator if( $$data{$option} ne "" );
            $$data{$option} .= sprintf( $format, @vals );
            }
        }
    close( NPA );
}

# Args:   1: Canonical Name
# Return:    ( lpr-status, Queueinhalt, printing enabled, spooling enabled )

sub get_lpr_info
{
    my $f;
    my $name = $_[0];
    my $stat, $queue; 
    
    $p = $hosts->{$name};
        
    $f=0;
    open( LPQ, "$LPQ -P$name|" ) || die "Can't start lpq";
    while( <LPQ> ){
        $stat = "$1" if( ( !$f ) && /^ Status: (.*)$/ );
        $f = 1 if( ( !$f ) && /^ Rank/ );
        $queue .= "$_" if( $f );
        }
    close( LPQ );

    open( LPC, "$LPC status $name 2>&1|" ) || die "Can't start lpc";
    <LPC>;
    $_ = <LPC>;
    close( LPC );
    /^\S+\s+(\S+)\s+(\S+)/;
    
    return( $stat, $queue, "$1" eq "enabled", "$2" eq "enabled" );
}

sub read_printer_dat
{
    my $hidden = 0;

    open( DAT, "$PRINTER_DAT_PATH" ) || die "Can't open $PRINTER_DAT_PATH";
    while( <DAT> ){

        $hidden = 1 if( /^###/ );
        next if( /^$/ || /^##/ );
        push( @PRINTER_DAT, $_ ), next if( /^#/ );
        
        split;
        push( @PRINTER_DAT, $_[0] );
        $PRINTER_DAT{ "$_[0]" } = [ $_[1], $_[2], $_[3], $_[4], $_[5], $_[6], $_[7], $_[8], $_[9], $hidden ];

        }
        
    close( <DAT> );        
    
}

read_printer_dat();
true;
    

    
